#include <WiFi.h>

//SCL - 22; SDA - 21;

#include <Wire.h>
#include <Adafruit_BME280.h>
#include <Adafruit_Sensor.h>

#include <Adafruit_I2CDevice.h>


#include <HardwareSerial.h>

#define SEALEVELPRESSURE_HPA (1013.25)

Adafruit_BME280 bme;

#define DEBUG false

// Replace with your network credentials
const char* ssid = "SSID"; // REPLACE WITH YOUR SSID
const char* password = "PASSWORD"; // REPLACE WITH YOUR PASSWORD

// Set web server port number to 80
WiFiServer server(80);

// Variable to store the HTTP request
String header;

String RMC;
String ReadString;

// Current time
unsigned long currentTime = millis();
// Previous time
unsigned long previousTime = 0; 
// Define timeout time in milliseconds (example: 2000ms = 2s)
const long timeoutTime = 2000;

bool isWaiting(String RMCdata) {
  String stat = getChunkValue(RMCdata, ',', 2);
  if (stat == "A") {
    return false;
  }
  return true;
}

String getDateFromRMC(String RMCdata) {
  String dateData = getChunkValue(RMCdata, ',', 9);
  if (dateData.length() > 0) {
    String dayStr = dateData.substring(0, 2);
    String monthStr = dateData.substring(2, 4);
    String yearStr = dateData.substring(4, 6);
    return dayStr + "." + monthStr + "." + yearStr;
  }
  return "Waiting...";
}

String getTimeFromRMC(String RMCdata) {
  String timeData = getChunkValue(RMCdata, ',', 1);
  if (timeData.length() > 0) {
    String hours = timeData.substring(0, 2);
    String minutes = timeData.substring(2, 4);
    String seconds = timeData.substring(4, 6);
    return hours + ":" + minutes + ":" + seconds;
  }
  return "Waiting...";
}

String getLatFromRMC(String RMCdata) {
  String lat = getChunkValue(RMCdata, ',', 3);
  String indicator = getChunkValue(RMCdata, ',', 4);
  return getPosition(lat, indicator);
}

String getLonFromRMC(String RMCdata) {
  String lon = getChunkValue(RMCdata, ',', 5);
  String indicator = getChunkValue(RMCdata, ',', 6);
  return getPosition(lon, indicator);
}

int getSpeedFromRMC(String RMCdata) {
  String speedData = getChunkValue(RMCdata, ',', 7);
  return int(speedData.toFloat() / float(0.53995680346));
}

String getPosition(String data, String indicator) {
  int D = int(data.toFloat() / float(100));
  float M = data.toFloat() - float(D) * float(100);
  float pos = D + M/60;
  if (indicator == "S" or indicator == "W") {
    pos = pos * -1;
  }
  return String(pos, 6);
}

String getChunkValue(String data, char separator, int index) {
  int maxIndex = data.length() - 1;
  int j = 0;
  String chunkVal = "";
  for (int i = 0; i <= maxIndex && j <= index; i++) {
    chunkVal.concat(data[i]);
    if (data[i] == separator) {
      j++;
      if (j > index) {
        chunkVal.trim();
        return chunkVal.substring(0, chunkVal.length() - 1);
      }
      chunkVal = "";
    }
    else if ((i == maxIndex) && (j < index)) {
      chunkVal = "";
      return chunkVal;
    }
  }
}

void setup() {
  Serial.begin(115200);
  // Initialize the output variables as outputs
  /*
  * UART1  -> Serial1
  * RX Pin -> GPIO 14
  * TX Pin -> GPIO 12
  * UART Configuration -> SERIAL_8N1
  */
  Serial1.begin(9600, SERIAL_8N1, 14, 12);

  bool status;

  // default settings
  // (you can also pass in a Wire library object like &Wire2)
  //status = bme.begin();  
  if (!bme.begin(0x76)) {
    Serial.println("Could not find a valid BME280 sensor, check wiring!");
    while (1);
  }
  
  // Connect to Wi-Fi network with SSID and password
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  // Print local IP address and start web server
  Serial.println("");
  Serial.println("WiFi connected.");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  server.begin();
}

void loop(){
  WiFiClient client = server.available();   // Listen for incoming clients

  ReadString=Serial1.readStringUntil('\r');
  ReadString.trim();
  if (ReadString.startsWith("$GPRMC") or ReadString.startsWith("$GNRMC")) {
    RMC = ReadString;
  }
  

  if (client) {                             // If a new client connects,
    currentTime = millis();
    previousTime = currentTime;
    
    

            
    if (DEBUG) {
      Serial.println(ReadString);
    }
    
    Serial.println("New Client.");          // print a message out in the serial port
    String currentLine = "";                // make a String to hold incoming data from the client
    while (client.connected() && currentTime - previousTime <= timeoutTime) {  // loop while the client's connected
      
      
      currentTime = millis();
      if (client.available()) {             // if there's bytes to read from the client,
        char c = client.read();             // read a byte, then
        Serial.write(c);                    // print it out the serial monitor
        header += c;
        if (c == '\n') {                    // if the byte is a newline character
          // if the current line is blank, you got two newline characters in a row.
          // that's the end of the client HTTP request, so send a response:
          if (currentLine.length() == 0) {
            // HTTP headers always start with a response code (e.g. HTTP/1.1 200 OK)
            // and a content-type so the client knows what's coming, then a blank line:
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println("Connection: close");
            client.println();


            
            
            // turns the GPIOs on and off

            
            // Display the HTML web page
            client.println("<!DOCTYPE html><html>");
            client.println("<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">");
            client.println("<link rel=\"icon\" href=\"data:,\">");
            // CSS to style the on/off buttons 
            // Feel free to change the background-color and font-size attributes to fit your preferences

            
            // Web Page Heading
            //client.println("<body><h1>LOCATION + </h1>");

            /*client.println("<h1>Date UTC = " + getDateFromRMC(RMC) + "</h1>");
            client.println("<h1>Time UTC = " + getTimeFromRMC(RMC) + "</h1>");
            client.println("<h1>Lat = " + getLatFromRMC(RMC) + "</h1>");
            client.println("<h1>Lon = " + getLonFromRMC(RMC) + "</h1>");
            client.println("<h1>Speed km/h = " + String(getSpeedFromRMC(RMC)) + "</h1>");*/

            client.println("<style>body { text-align: center; font-family: \"Trebuchet MS\", Arial;}");
            client.println("table { border-collapse: collapse; width:35%; margin-left:auto; margin-right:auto; }");
            client.println("th { padding: 12px; background-color: #0043af; color: white; }");
            client.println("tr { border: 1px solid #ddd; padding: 12px; }");
            client.println("tr:hover { background-color: #bcbcbc; }");
            client.println("td { border: none; padding: 12px; }");
            client.println(".sensor { color:white; font-weight: bold; background-color: #bcbcbc; padding: 1px; }");
            
            // Web Page Heading
            client.println("</style></head><body><h1>ESP32 Web Server</h1>");
            client.println("<table><tr><th>MEASUREMENT</th><th>VALUE</th></tr>");

            client.println("<tr><td>Date UTC</td><td><span class=\"sensor\">");
            client.println(getDateFromRMC(RMC));
            client.println("</span></td></tr>");

            client.println("<tr><td>Time UTC</td><td><span class=\"sensor\">");
            client.println(getTimeFromRMC(RMC));
            client.println("</span></td></tr>");

            client.println("<tr><td>Lat</td><td><span class=\"sensor\">");
            client.println(getLatFromRMC(RMC));
            client.println("</span></td></tr>");

            client.println("<tr><td>Lon</td><td><span class=\"sensor\">");
            client.println(getLonFromRMC(RMC));
            client.println("</span></td></tr>");

            client.println("<tr><td>Speed</td><td><span class=\"sensor\">");
            client.println(String(getSpeedFromRMC(RMC)));
            client.println("km/h</span></td></tr>");

            client.println("<tr><td>Temp. Celsius</td><td><span class=\"sensor\">");
            client.println(bme.readTemperature());
            client.println(" *C</span></td></tr>");  
            client.println("<tr><td>Temp. Fahrenheit</td><td><span class=\"sensor\">");
            client.println(1.8 * bme.readTemperature() + 32);
            client.println(" *F</span></td></tr>");       
            client.println("<tr><td>Pressure</td><td><span class=\"sensor\">");
            client.println(bme.readPressure() / 100.0F);
            client.println(" hPa</span></td></tr>");
            client.println("<tr><td>Approx. Altitude</td><td><span class=\"sensor\">");
            client.println(bme.readAltitude(SEALEVELPRESSURE_HPA));
            client.println(" m</span></td></tr>"); 
            client.println("<tr><td>Humidity</td><td><span class=\"sensor\">");
            client.println(bme.readHumidity());
            client.println(" %</span></td></tr>"); 
            client.println("</body></html>");

            

            // If the output26State is off, it displays the ON button       

            // Display current state, and ON/OFF buttons for GPIO 27  

            client.println("</body></html>");
            
            // The HTTP response ends with another blank line
            client.println();
            // Break out of the while loop
            break;
          } else { // if you got a newline, then clear currentLine
            currentLine = "";
          }
        } else if (c != '\r') {  // if you got anything else but a carriage return character,
          currentLine += c;      // add it to the end of the currentLine
        }
      }
    }
    // Clear the header variable
    header = "";
    // Close the connection
    client.stop();
    Serial.println("Client disconnected.");
    Serial.println("");
  }
}
